# -*- coding: utf-8 -*-
import json
import sentiment_analysis_functions as sfun
from sklearn.metrics import accuracy_score, recall_score, precision_score
from scipy.sparse import csr_matrix, hstack
import pandas as pd

# installation
# import nltk
# nltk.download('punkt')
# nltk.download('wordnet')
# nltk.download('stopwords')
# nltk.download('averaged_perceptron_tagger')
# nltk.download('universal_tagset')
# nltk.download('sentiwordnet')


train_pos_reviews_fn = "./data/train-positive-t1.txt"
train_neg_reviews_fn = "./data/train-negative-t1.txt"
test_pos_reviews_fn = "./data/test-pos-t1.txt"
test_neg_reviews_fn = "./data/test-neg-t1.txt"


# Aucune contrainte pour cette tâche. Vous pouvez structurez votre code comme bon vous semble.
# Expliquez dans votre rapport comment exécuter le code et définir les configurations.


def load_reviews(filename):
    with open(filename, 'r') as fp:
        reviews_list = json.load(fp)
    return reviews_list


if __name__ == '__main__':
    train_positive_reviews = load_reviews(train_pos_reviews_fn)
    train_negative_reviews = load_reviews(train_neg_reviews_fn)
    test_positive_reviews = load_reviews(test_pos_reviews_fn)
    test_negative_reviews = load_reviews(test_neg_reviews_fn)
    print("Nb of training reviews - positive:", len(train_positive_reviews), "negative:", len(train_negative_reviews))
    print("Nb of test reviews - positive:", len(test_positive_reviews), "negative:", len(test_negative_reviews))
    
    # Train and test datasets
    
    train_dataset = train_positive_reviews+train_negative_reviews
    train_dataset_response = [1]*len(train_positive_reviews)+[0]*len(train_negative_reviews)
    
    test_dataset = test_positive_reviews+test_negative_reviews
    test_dataset_response = [1]*len(test_positive_reviews)+[0]*len(test_negative_reviews)
    
    # Méthodes
    
    norm_names = {
        0:"Stemm.",
        1:"Lemm."
    }
    
    select_names = {
        0:"Fréq.",
        1:"M. outil",
        2:"Cl. ouv."
    }
    
    vectorizer_names = {
        0:"Compt.",
        1:"Occ.",
        2:"TF-IDF"
    }

    # Tokenisation
    
    # Tokenize train
    train_tokens = sfun.tokenize(train_dataset)
    norm_train_tokens = []
    norm_train_tokens.append(sfun.norm_stemming(train_tokens))
    norm_train_tokens.append(sfun.norm_lemmatize(train_tokens))
    
    # Tokenize test
    test_tokens = sfun.tokenize(test_dataset)
    norm_test_tokens = []
    norm_test_tokens.append(sfun.norm_stemming(test_tokens))
    norm_test_tokens.append(sfun.norm_lemmatize(test_tokens))   
    
    # Normalize and select tokens
    norm_select_train_tokens = []
    norm_select_test_tokens = []
    norm_select_train_tokens_split = []
    norm_select_test_tokens_split = []
    for norm_m in range(0,2):
        # train tokens
        select_train_tokens = []
        select_train_tokens_split = []
        nn1 = sfun.select_freq(norm_train_tokens[norm_m])
        nn2 = sfun.select_rem_stopwords(norm_train_tokens[norm_m])
        nn3 = sfun.select_open_class(norm_train_tokens[norm_m])
        select_train_tokens_split.append(nn1)
        select_train_tokens_split.append(nn2)
        select_train_tokens_split.append(nn3)
        select_train_tokens.append(sfun.corpus_documents(nn1))
        select_train_tokens.append(sfun.corpus_documents(nn2))
        select_train_tokens.append(sfun.corpus_documents(nn3))       
        norm_select_train_tokens_split.append(select_train_tokens_split)
        norm_select_train_tokens.append(select_train_tokens)
        # test tokens
        select_test_tokens = []
        select_test_tokens_split = []
        nn1 = sfun.select_freq(norm_test_tokens[norm_m])
        nn2 = sfun.select_rem_stopwords(norm_test_tokens[norm_m])
        nn3 = sfun.select_open_class(norm_test_tokens[norm_m])
        select_test_tokens_split.append(nn1)
        select_test_tokens_split.append(nn2)
        select_test_tokens_split.append(nn3)
        select_test_tokens.append(sfun.corpus_documents(nn1))
        select_test_tokens.append(sfun.corpus_documents(nn2))
        select_test_tokens.append(sfun.corpus_documents(nn3))       
        norm_select_test_tokens_split.append(select_test_tokens_split)
        norm_select_test_tokens.append(select_test_tokens)
    
    # Création des objets Vectorizer (Train seulement)
    norm_select_vectorizers = []
    for norm_m in range(0,2):
        select_vectorizers = []
        for sel_m in range(0,3):
            vectorizers = sfun.get_vectorizers(norm_select_train_tokens[norm_m][sel_m])
            select_vectorizers.append(vectorizers)
        norm_select_vectorizers.append(select_vectorizers)
    
    # Transformation des jetons en vecteurs
    v_norm_select_vectors_train = []
    v_norm_select_vectors_test = []
    for norm_m in range(0,2):
        v_select_vectors_train = []
        v_select_vectors_test = []        
        for sel_m in range(0,3):
            v_vectors_train = []
            v_vectors_test = []
            for vect_m in range(0,3):
                v_vector_train = norm_select_vectorizers[norm_m][sel_m][vect_m].transform(norm_select_train_tokens[norm_m][sel_m])
                v_vector_test = norm_select_vectorizers[norm_m][sel_m][vect_m].transform(norm_select_test_tokens[norm_m][sel_m])
                v_vectors_train.append(v_vector_train)
                v_vectors_test.append(v_vector_test)
            v_select_vectors_train.append(v_vectors_train)
            v_select_vectors_test.append(v_vectors_test)
        v_norm_select_vectors_train.append(v_select_vectors_train)
        v_norm_select_vectors_test.append(v_select_vectors_test)
        
    # Ajout des attributs
    v_norm_select_polarity_count_train = []
    v_norm_select_polarity_count_test = []
    for norm_m in range(0,2):
        v_select_polarity_count_train = []
        v_select_polarity_count_test = []
        for sel_m in range(0,3):
            v_polarity_count_train = sfun.attribute_polarity_count(norm_select_train_tokens_split[norm_m][sel_m])
            v_polarity_count_test = sfun.attribute_polarity_count(norm_select_test_tokens_split[norm_m][sel_m])
            v_select_polarity_count_train.append(csr_matrix(v_polarity_count_train))
            v_select_polarity_count_test.append(csr_matrix(v_polarity_count_test))
        v_norm_select_polarity_count_train.append(v_select_polarity_count_train)
        v_norm_select_polarity_count_test.append(v_select_polarity_count_test)
    
    # Création des matrices finales
    v_final_train = []
    v_final_test = []
    for norm_m in range(0,2):
        v_select_final_train = []
        v_select_final_test = []
        for sel_m in range(0,3):
            v_vector_final_train = []
            v_vector_final_test = []            
            for vect_m in range(0,3):
                v_vector_final_train.append(hstack([v_norm_select_vectors_train[norm_m][sel_m][vect_m],
                                                v_norm_select_polarity_count_train[norm_m][sel_m]]))
                v_vector_final_test.append(hstack([v_norm_select_vectors_test[norm_m][sel_m][vect_m],
                                               v_norm_select_polarity_count_test[norm_m][sel_m]]))
            v_select_final_train.append(v_vector_final_train)
            v_select_final_test.append(v_vector_final_test)
        v_final_train.append(v_select_final_train)
        v_final_test.append(v_select_final_test)
            
    # Scoring des modèles
    
    modeles_nb = []
    scores_nb = []
    modeles_reg = []
    scores_reg = []
    for norm_m in range(0,2):
        modeles_select_vector_nb = []
        scores_select_vector_nb = []
        modeles_select_vector_reg = []
        scores_select_vector_reg = []
        for sel_m in range(0,3):
            modeles_vector_nb = []
            scores_vector_nb = []
            modeles_vector_reg = []
            scores_vector_reg = []
            for vect_m in range(0,3):
                modele_nb = sfun.train_naive_model(v_final_train[norm_m][sel_m][vect_m],train_dataset_response)
                score_nb = modele_nb.predict(v_final_test[norm_m][sel_m][vect_m])
                modele_reg = sfun.train_regression_model(v_final_train[norm_m][sel_m][vect_m],train_dataset_response)
                score_reg = modele_reg.predict(v_final_test[norm_m][sel_m][vect_m])
                modeles_vector_nb.append(modele_nb)
                scores_vector_nb.append(score_nb)
                modeles_vector_reg.append(modele_reg)
                scores_vector_reg.append(score_reg)
            modeles_select_vector_nb.append(modeles_vector_nb)
            scores_select_vector_nb.append(scores_vector_nb)
            modeles_select_vector_reg.append(modeles_vector_reg)
            scores_select_vector_reg.append(scores_vector_reg)
        modeles_nb.append(modeles_select_vector_nb)
        scores_nb.append(scores_select_vector_nb)
        modeles_reg.append(modeles_select_vector_reg)
        scores_reg.append(scores_select_vector_reg)
        
    # Performance des modéles
    
    def class_metrics(test_y,pred_y):
        return [precision_score(test_y,pred_y),recall_score(test_y,pred_y),accuracy_score(test_y,pred_y)]
    
    cm_nb = []
    cm_reg = []
    for norm_m in range(0,2):
        cm_sel_vect_nb = []
        cm_sel_vect_reg = []
        for sel_m in range(0,3):
            cm_vect_nb = []
            cm_vect_reg = []
            for vect_m in range(0,3):
                cm_vect_nb.append(class_metrics(test_dataset_response,scores_nb[norm_m][sel_m][vect_m]))
                cm_vect_reg.append(class_metrics(test_dataset_response,scores_reg[norm_m][sel_m][vect_m]))
            cm_sel_vect_nb.append(cm_vect_nb)
            cm_sel_vect_reg.append(cm_vect_reg)
        cm_nb.append(cm_sel_vect_nb)
        cm_reg.append(cm_sel_vect_reg)
     
    table_metriques = []
    for norm_m in range(0,2):
        for sel_m in range(0,3):
            for vect_m in range(0,3):
                table_metriques.append([norm_names.get(norm_m),
                                        select_names.get(sel_m),
                                        vectorizer_names.get(vect_m)]+
    cm_nb[norm_m][sel_m][vect_m]+
    cm_reg[norm_m][sel_m][vect_m])
                
    table_metriques_df = pd.DataFrame(table_metriques,columns=['Norm.',
                                                               'Sélect.',
                                                               'Vect.',
                                                               'Préc. NB',
                                                               'Rappel NB',
                                                               'Exac. NB',
                                                               'Préc. RL',
                                                               'Rappel RL',
                                                               'Exac. RL'])
    
    # Tableau des métrique Naive Bayes
    
    f = open("table_metriques_nb.tex","w")
    f.write(
    r"""
    \documentclass[preview]{standalone}
    \usepackage{booktabs}
    \begin{document}
    """)
    f.write(table_metriques_df[['Norm.',
                                'Sélect.',
                                'Vect.',
                                'Préc. NB',
                                'Rappel NB',
                                'Exac. NB']].to_latex())
    f.write(
    r"""
    \end{document}
    """)
    f.close()
 
    # Tableau des métriques Régression logistique
    
    f = open("table_metriques_rl.tex","w")
    f.write(
    r"""
    \documentclass[preview]{standalone}
    \usepackage{booktabs}
    \begin{document}
    """)
    f.write(table_metriques_df[['Norm.',
                                'Sélect.',
                                'Vect.',
                                'Préc. RL',
                                'Rappel RL',
                                'Exac. RL']].to_latex())
    f.write(
    r"""
    \end{document}
    """)
    f.close()
